﻿using C1.Win.C1FlexGrid;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CrossHair_C1FlexGrid
{
    public partial class Form1 : Form
    {
        int prevRow;
        int prevCol;
        bool flag = false;
        SolidBrush _bdrBrush;
        Timer timer;
        public Form1()
        {
            InitializeComponent();
            prevRow= -1;
            prevCol= -1;

            _bdrBrush = new SolidBrush(Color.Red);

            c1FlexGrid1.Font = new Font(c1FlexGrid1.Font.FontFamily, 10, FontStyle.Regular);
            c1FlexGrid1.Rows[0].TextAlign = TextAlignEnum.CenterCenter;
            c1FlexGrid1.Rows[0].Style.Font = new Font(c1FlexGrid1.Rows[0].Style.Font.FontFamily,11, FontStyle.Bold);

            c1FlexGrid1.CellChanged += C1FlexGrid1_CellChanged;
            c1FlexGrid1.DrawMode = C1.Win.C1FlexGrid.DrawModeEnum.OwnerDraw;
            c1FlexGrid1.OwnerDrawCell += C1FlexGrid1_OwnerDrawCell;
        }
        private void C1FlexGrid1_OwnerDrawCell(object sender, C1.Win.C1FlexGrid.OwnerDrawCellEventArgs e)
        {
            if(!e.Measuring)
            {
                // we only want cells with our style  
                CellStyle style = c1FlexGrid1.GetCellStyle(e.Row, e.Col);
                if (style == null || (style.Name != "HorizontalBorder" && style.Name != "VerticalBorder" && style.Name!= "MergedBorder"))
                {
                    return;
                }

                // draw cell content as usual
                e.DrawCell();
                Graphics gr;
                Rectangle rec;

                //if cell resides in row whose cell changed
                if (style.Name == "HorizontalBorder")
                {
                    Margins marginHz = new Margins(0,0,3,0);
                    gr = e.Graphics;
                    rec = e.Bounds;
                    rec.Height = marginHz.Top;
                    gr.FillRectangle(_bdrBrush, rec);
                    prevRow = e.Row;
                }
                //if cell resides in column whose cell changed
                else if (style.Name == "VerticalBorder")
                {
                    Margins marginVr = new Margins(3, 0, 0,0);
                    gr = e.Graphics;
                    rec = e.Bounds;
                    rec.Width = marginVr.Left;
                    gr.FillRectangle(_bdrBrush, rec);
                    prevCol = e.Col;
                }
                //if cell resides in row and column (both) whose cell changed
                else if (style.Name== "MergedBorder")
                {
                    Margins marginMrg = new Margins(3, 0, 3,0);
                    gr = e.Graphics;
                    rec = e.Bounds;
                    rec.Height = marginMrg.Top;
                    gr.FillRectangle(_bdrBrush, rec);

                    rec = e.Bounds;
                    rec.Width = marginMrg.Left;
                    gr.FillRectangle(_bdrBrush, rec);
                }
            }
        }
        private void C1FlexGrid1_CellChanged(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            if (prevRow > -1)
            {
                CellRange prevRowRange = c1FlexGrid1.GetCellRange(prevRow, 1, prevRow, c1FlexGrid1.Cols.Count - 1);
                prevRowRange.Style = null;
                prevRow = -1;
            }
            if(prevCol>-1)
            {
                CellRange prevColRange = c1FlexGrid1.GetCellRange(1, prevCol, c1FlexGrid1.Rows.Count - 1, prevCol);
                prevColRange.Style = null;
                prevCol = -1;

            }
            //getting all cells in modified column
            CellRange colRange = c1FlexGrid1.GetCellRange(1,e.Col,c1FlexGrid1.Rows.Count-1,e.Col);  
            colRange.Style = c1FlexGrid1.Styles["VerticalBorder"];

            //getting all cells in modified row
            CellRange rowRange = c1FlexGrid1.GetCellRange(e.Row,  1, e.Row, c1FlexGrid1.Cols.Count - 1); 
            rowRange.Style = c1FlexGrid1.Styles["HorizontalBorder"];

            //getting the intersection cell[modifiedRow,modifiedCol]
            CellStyle mergedStyle = c1FlexGrid1.Styles["MergedBorder"]; 
            c1FlexGrid1.SetCellStyle(e.Row, e.Col, mergedStyle);

            //showing the modified cell in view
            c1FlexGrid1.TopRow = e.Row - c1FlexGrid1.TopRow;   
            c1FlexGrid1.Select(e.Row, e.Col);
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            c1FlexGrid1.DataSource = FinancialData.GetFinancialData().Take(50).ToList();
            c1FlexGrid1.AutoSizeRow(0);
            timer = new Timer();
            timer.Tick += timer_Tick;
            timer.Start();

            comboBox1.SelectedIndex = 1;

            #region "Formatting Grid columns"
            c1FlexGrid1.Cols["Bid"].Format = "0.00";
            c1FlexGrid1.Cols["Ask"].Format = "0.00";
            c1FlexGrid1.Cols["Ask"].Move(c1FlexGrid1.Cols["Ask"].Index + 1);

            c1FlexGrid1.AutoSizeCol(2);
            foreach (Column c in c1FlexGrid1.Cols)
            {
                c.TextAlign = TextAlignEnum.CenterCenter;
            }

            #endregion

            //adding Vertical style to FlexGrid
            CellStyle vStyle = c1FlexGrid1.Styles.Add("VerticalBorder");

            //adding Horizontal style to FlexGrid
            CellStyle hStyle = c1FlexGrid1.Styles.Add("HorizontalBorder");

            // adding style where Vertical and Horizontal style cross
            CellStyle mergedStyle = c1FlexGrid1.Styles.Add("MergedBorder"); 
        }
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (FinancialData.GetFinancialData() != null)
            {
                var cmb = sender as ComboBox;
                var val = cmb.SelectedItem as string;
                val = val.Split(' ')[0].Trim();
                timer.Interval = int.Parse(val);
            }
        }
        Random _rnd = new Random(0);
        void timer_Tick(object sender, EventArgs e)
        {               
            int index = _rnd.Next() % 50;
            if(index>0)
            {
                if(flag)
                {
                    c1FlexGrid1[index, "Bid"] = Convert.ToInt32(c1FlexGrid1[index, "Bid"]) * (decimal)(1 + (_rnd.NextDouble() * .11 - .05));
                    flag = !flag;
                }
                else
                {
                    c1FlexGrid1[index, "Ask"] = Convert.ToInt32(c1FlexGrid1[index, "Ask"]) * (decimal)(1 + (_rnd.NextDouble() * .11 - .05));
                    flag = !flag;
                }
            }
        }
    }
}
